/********************************************************************************
** Form generated from reading UI file 'imagepropertiesdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEPROPERTIESDIALOG_H
#define UI_IMAGEPROPERTIESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_imagePropertiesDialog
{
public:
    QVBoxLayout *mainLayout;
    QTabWidget *tabWidget;
    QWidget *generalTab;
    QFormLayout *formLayout;
    QLabel *label;
    QLabel *imageNameLabel;
    QLabel *label_2;
    QLabel *folderLabel;
    QLabel *label_3;
    QLabel *sizeLabel;
    QLabel *label_4;
    QLabel *colorCountLabel;
    QLabel *label_9;
    QLabel *fileTypeLabel;
    QLabel *label_5;
    QLabel *fileSizeLabel;
    QLabel *label_6;
    QLabel *dateLabel;
    QLabel *label_7;
    QLabel *memorySizeLabel;
    QLabel *label_8;
    QLabel *totalSizeLabel;
    QWidget *exifTab;
    QVBoxLayout *exifLayout;
    QTableWidget *exifTable;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *imagePropertiesDialog)
    {
        if (imagePropertiesDialog->objectName().isEmpty())
            imagePropertiesDialog->setObjectName("imagePropertiesDialog");
        imagePropertiesDialog->setWindowModality(Qt::ApplicationModal);
        imagePropertiesDialog->resize(560, 380);
        mainLayout = new QVBoxLayout(imagePropertiesDialog);
        mainLayout->setObjectName("mainLayout");
        tabWidget = new QTabWidget(imagePropertiesDialog);
        tabWidget->setObjectName("tabWidget");
        generalTab = new QWidget();
        generalTab->setObjectName("generalTab");
        formLayout = new QFormLayout(generalTab);
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        formLayout->setLabelAlignment(Qt::AlignRight|Qt::AlignVCenter);
        formLayout->setHorizontalSpacing(8);
        formLayout->setVerticalSpacing(6);
        label = new QLabel(generalTab);
        label->setObjectName("label");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label);

        imageNameLabel = new QLabel(generalTab);
        imageNameLabel->setObjectName("imageNameLabel");
        imageNameLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, imageNameLabel);

        label_2 = new QLabel(generalTab);
        label_2->setObjectName("label_2");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, label_2);

        folderLabel = new QLabel(generalTab);
        folderLabel->setObjectName("folderLabel");
        folderLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, folderLabel);

        label_3 = new QLabel(generalTab);
        label_3->setObjectName("label_3");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, label_3);

        sizeLabel = new QLabel(generalTab);
        sizeLabel->setObjectName("sizeLabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, sizeLabel);

        label_4 = new QLabel(generalTab);
        label_4->setObjectName("label_4");

        formLayout->setWidget(3, QFormLayout::ItemRole::LabelRole, label_4);

        colorCountLabel = new QLabel(generalTab);
        colorCountLabel->setObjectName("colorCountLabel");

        formLayout->setWidget(3, QFormLayout::ItemRole::FieldRole, colorCountLabel);

        label_9 = new QLabel(generalTab);
        label_9->setObjectName("label_9");

        formLayout->setWidget(4, QFormLayout::ItemRole::LabelRole, label_9);

        fileTypeLabel = new QLabel(generalTab);
        fileTypeLabel->setObjectName("fileTypeLabel");

        formLayout->setWidget(4, QFormLayout::ItemRole::FieldRole, fileTypeLabel);

        label_5 = new QLabel(generalTab);
        label_5->setObjectName("label_5");

        formLayout->setWidget(5, QFormLayout::ItemRole::LabelRole, label_5);

        fileSizeLabel = new QLabel(generalTab);
        fileSizeLabel->setObjectName("fileSizeLabel");

        formLayout->setWidget(5, QFormLayout::ItemRole::FieldRole, fileSizeLabel);

        label_6 = new QLabel(generalTab);
        label_6->setObjectName("label_6");

        formLayout->setWidget(6, QFormLayout::ItemRole::LabelRole, label_6);

        dateLabel = new QLabel(generalTab);
        dateLabel->setObjectName("dateLabel");

        formLayout->setWidget(6, QFormLayout::ItemRole::FieldRole, dateLabel);

        label_7 = new QLabel(generalTab);
        label_7->setObjectName("label_7");

        formLayout->setWidget(7, QFormLayout::ItemRole::LabelRole, label_7);

        memorySizeLabel = new QLabel(generalTab);
        memorySizeLabel->setObjectName("memorySizeLabel");

        formLayout->setWidget(7, QFormLayout::ItemRole::FieldRole, memorySizeLabel);

        label_8 = new QLabel(generalTab);
        label_8->setObjectName("label_8");

        formLayout->setWidget(8, QFormLayout::ItemRole::LabelRole, label_8);

        totalSizeLabel = new QLabel(generalTab);
        totalSizeLabel->setObjectName("totalSizeLabel");

        formLayout->setWidget(8, QFormLayout::ItemRole::FieldRole, totalSizeLabel);

        tabWidget->addTab(generalTab, QString());
        exifTab = new QWidget();
        exifTab->setObjectName("exifTab");
        exifLayout = new QVBoxLayout(exifTab);
        exifLayout->setObjectName("exifLayout");
        exifTable = new QTableWidget(exifTab);
        if (exifTable->columnCount() < 2)
            exifTable->setColumnCount(2);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        exifTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        exifTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        exifTable->setObjectName("exifTable");
        exifTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        exifTable->setAlternatingRowColors(true);
        exifTable->setSelectionBehavior(QAbstractItemView::SelectRows);
        exifTable->setShowGrid(false);
        exifTable->setColumnCount(2);
        exifTable->horizontalHeader()->setStretchLastSection(true);

        exifLayout->addWidget(exifTable);

        tabWidget->addTab(exifTab, QString());

        mainLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(imagePropertiesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        mainLayout->addWidget(buttonBox);


        retranslateUi(imagePropertiesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, imagePropertiesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, imagePropertiesDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(imagePropertiesDialog);
    } // setupUi

    void retranslateUi(QDialog *imagePropertiesDialog)
    {
        imagePropertiesDialog->setWindowTitle(QCoreApplication::translate("imagePropertiesDialog", "Image Properties", nullptr));
        label->setText(QCoreApplication::translate("imagePropertiesDialog", "Image name:", nullptr));
        imageNameLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_2->setText(QCoreApplication::translate("imagePropertiesDialog", "Folder:", nullptr));
        folderLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_3->setText(QCoreApplication::translate("imagePropertiesDialog", "Size:", nullptr));
        sizeLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_4->setText(QCoreApplication::translate("imagePropertiesDialog", "Color count:", nullptr));
        colorCountLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_9->setText(QCoreApplication::translate("imagePropertiesDialog", "File type:", nullptr));
        fileTypeLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_5->setText(QCoreApplication::translate("imagePropertiesDialog", "File size:", nullptr));
        fileSizeLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_6->setText(QCoreApplication::translate("imagePropertiesDialog", "Date:", nullptr));
        dateLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_7->setText(QCoreApplication::translate("imagePropertiesDialog", "Memory size:", nullptr));
        memorySizeLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        label_8->setText(QCoreApplication::translate("imagePropertiesDialog", "Undo cache size:", nullptr));
        totalSizeLabel->setText(QCoreApplication::translate("imagePropertiesDialog", "TextLabel", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(generalTab), QCoreApplication::translate("imagePropertiesDialog", "General", nullptr));
        QTableWidgetItem *___qtablewidgetitem = exifTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("imagePropertiesDialog", "Property", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = exifTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("imagePropertiesDialog", "Value", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(exifTab), QCoreApplication::translate("imagePropertiesDialog", "EXIF", nullptr));
    } // retranslateUi

};

namespace Ui {
    class imagePropertiesDialog: public Ui_imagePropertiesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEPROPERTIESDIALOG_H
