/********************************************************************************
** Form generated from reading UI file 'StampSettingsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STAMPSETTINGSWIDGET_H
#define UI_STAMPSETTINGSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_StampSettingsWidget
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label_3;
    QSpinBox *radiusSpinBox;
    QVBoxLayout *verticalLayout_3;
    QLabel *label;
    QSlider *pressureSlider;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_2;
    QSlider *stepSlider;
    QCheckBox *fixedCheckBox;
    QCheckBox *preciseCheckBox;
    QCheckBox *diffuseCheckBox;

    void setupUi(QWidget *StampSettingsWidget)
    {
        if (StampSettingsWidget->objectName().isEmpty())
            StampSettingsWidget->setObjectName("StampSettingsWidget");
        StampSettingsWidget->resize(134, 280);
        gridLayout = new QGridLayout(StampSettingsWidget);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(-1, 0, -1, -1);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        label_3 = new QLabel(StampSettingsWidget);
        label_3->setObjectName("label_3");

        verticalLayout->addWidget(label_3);

        radiusSpinBox = new QSpinBox(StampSettingsWidget);
        radiusSpinBox->setObjectName("radiusSpinBox");
        radiusSpinBox->setMinimumSize(QSize(0, 24));
        radiusSpinBox->setValue(32);

        verticalLayout->addWidget(radiusSpinBox);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        label = new QLabel(StampSettingsWidget);
        label->setObjectName("label");

        verticalLayout_3->addWidget(label);

        pressureSlider = new QSlider(StampSettingsWidget);
        pressureSlider->setObjectName("pressureSlider");
        pressureSlider->setMinimum(1);
        pressureSlider->setMaximum(10);
        pressureSlider->setPageStep(1);
        pressureSlider->setSliderPosition(10);
        pressureSlider->setOrientation(Qt::Orientation::Horizontal);
        pressureSlider->setTickPosition(QSlider::TickPosition::TicksBelow);

        verticalLayout_3->addWidget(pressureSlider);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        label_2 = new QLabel(StampSettingsWidget);
        label_2->setObjectName("label_2");

        verticalLayout_4->addWidget(label_2);

        stepSlider = new QSlider(StampSettingsWidget);
        stepSlider->setObjectName("stepSlider");
        stepSlider->setMinimum(1);
        stepSlider->setMaximum(99);
        stepSlider->setSingleStep(1);
        stepSlider->setPageStep(10);
        stepSlider->setOrientation(Qt::Orientation::Horizontal);
        stepSlider->setTickPosition(QSlider::TickPosition::TicksBelow);

        verticalLayout_4->addWidget(stepSlider);


        verticalLayout_3->addLayout(verticalLayout_4);


        verticalLayout->addLayout(verticalLayout_3);

        fixedCheckBox = new QCheckBox(StampSettingsWidget);
        fixedCheckBox->setObjectName("fixedCheckBox");

        verticalLayout->addWidget(fixedCheckBox);

        preciseCheckBox = new QCheckBox(StampSettingsWidget);
        preciseCheckBox->setObjectName("preciseCheckBox");

        verticalLayout->addWidget(preciseCheckBox);

        diffuseCheckBox = new QCheckBox(StampSettingsWidget);
        diffuseCheckBox->setObjectName("diffuseCheckBox");

        verticalLayout->addWidget(diffuseCheckBox);


        gridLayout->addLayout(verticalLayout, 0, 0, 1, 1);


        retranslateUi(StampSettingsWidget);

        QMetaObject::connectSlotsByName(StampSettingsWidget);
    } // setupUi

    void retranslateUi(QWidget *StampSettingsWidget)
    {
        StampSettingsWidget->setWindowTitle(QCoreApplication::translate("StampSettingsWidget", "Form", nullptr));
        label_3->setText(QCoreApplication::translate("StampSettingsWidget", "Radius", nullptr));
        label->setText(QCoreApplication::translate("StampSettingsWidget", "Pressure", nullptr));
        label_2->setText(QCoreApplication::translate("StampSettingsWidget", "Step", nullptr));
        fixedCheckBox->setText(QCoreApplication::translate("StampSettingsWidget", "Fixed", nullptr));
        preciseCheckBox->setText(QCoreApplication::translate("StampSettingsWidget", "Precise", nullptr));
        diffuseCheckBox->setText(QCoreApplication::translate("StampSettingsWidget", "Diffuse", nullptr));
    } // retranslateUi

};

namespace Ui {
    class StampSettingsWidget: public Ui_StampSettingsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STAMPSETTINGSWIDGET_H
